export declare type UnknownFunction = (...args: unknown[]) => unknown;
export declare type UnknownObject = Record<string, unknown>;
export declare type MaybeDate = Date | number | string;
export declare type Constructor<T> = (new (...args: unknown[]) => T) | (Function & {
    prototype: T;
});
export declare type InferNullable<P, N> = P extends null ? N | null : N;
export declare type DefaultValueInitializer<T> = (path: string, currentObject: UnknownObject, rootObject: UnknownObject) => T;
export declare type DefaultValue<T> = DefaultValueInitializer<T> | T;
export declare type ValueComparator<T> = (value: T, currentObject?: UnknownObject, rootObject?: UnknownObject) => boolean;
export declare type WhenCondition<T> = T | ValueComparator<T>;
export interface Options {
    message?: string;
}
export interface InclusiveOptions extends Options {
    inclusive?: boolean;
}
export declare type CriteriaValidator<Input> = (value: Input, path: string, options: SchemaValidateOptions) => unknown;
export interface Criteria<Input> {
    skipIfNull?: boolean;
    skipIfOptional?: boolean;
    validate: CriteriaValidator<Input>;
}
export declare type CriteriaFactory<Input> = (state: SchemaState<Input>, ...args: any[]) => Criteria<Input> | void;
export interface CommonCriterias<S> {
    and: (...keys: string[]) => S;
    custom: (callback: CriteriaValidator<InferSchemaType<S>>) => S;
    deprecate: (message: string) => S;
    notRequired: () => S;
    only: () => S;
    or: (...keys: string[]) => S;
    required: () => S;
    when: (condition: WhenCondition<InferSchemaType<S>>, pass: AnySchema, fail?: AnySchema) => S;
    xor: (...keys: string[]) => S;
}
export interface ArrayCriterias<S> {
    notEmpty: (options?: Options) => S;
    sizeOf: (size: number, options?: Options) => S;
}
export interface DateCriterias<S> {
    after: (date: MaybeDate, options?: Options) => S;
    before: (date: MaybeDate, options?: Options) => S;
    between: (start: MaybeDate, end: MaybeDate, options?: InclusiveOptions) => S;
}
export interface NumberCriterias<S> {
    between: (min: number, max: number, options?: InclusiveOptions) => S;
    float: (options?: Options) => S;
    gt: (min: number, options?: InclusiveOptions) => S;
    gte: (min: number, options?: Options) => S;
    int: (options?: Options) => S;
    lt: (max: number, options?: InclusiveOptions) => S;
    lte: (max: number, options?: Options) => S;
    negative: (options?: Options) => S;
    positive: (options?: Options) => S;
}
export interface ObjectCriterias<S> {
    notEmpty: (options?: Options) => S;
    sizeOf: (size: number, options?: Options) => S;
}
export interface ShapeCriterias<S> {
    exact: (state?: boolean) => S;
}
export interface StringCriterias<S> {
    camelCase: (options?: Options) => S;
    contains: (token: string, options?: Options & {
        index?: number;
    }) => S;
    kebabCase: (options?: Options) => S;
    lowerCase: (options?: Options) => S;
    match: (pattern: RegExp, options?: Options) => S;
    notEmpty: (options?: Options) => S;
    pascalCase: (options?: Options) => S;
    sizeOf: (size: number, options?: Options) => S;
    snakeCase: (options?: Options) => S;
    upperCase: (options?: Options) => S;
}
export interface SchemaValidateOptions {
    collectErrors?: boolean;
    currentObject?: UnknownObject;
    rootObject?: UnknownObject;
}
export interface Schema<Output, Input = Output> {
    schema: () => string;
    type: () => string;
    validate: (value: Input | null | undefined, path?: string, options?: SchemaValidateOptions) => Output;
}
export interface SchemaState<T> {
    defaultValue: DefaultValue<T> | undefined;
    metadata: UnknownObject;
    never: boolean;
    nullable: boolean;
    required: boolean;
    type: string;
}
export interface SchemaOptions<T> {
    api: Record<string, CriteriaFactory<T>>;
    cast?: (value: unknown) => T;
    defaultValue?: DefaultValue<T>;
    type: string;
}
export declare type InferSchemaType<T> = T extends Schema<infer U> ? U : never;
export declare type Blueprint<T extends object> = {
    [K in keyof T]-?: Schema<T[K]>;
};
export declare type Predicate<T> = (value: T | null | undefined) => boolean;
export declare type AnySchema = Schema<any, any>;
export declare type InferFromObject<T> = {
    [K in keyof T]: Infer<T[K]>;
};
export declare type Infer<T> = T extends Schema<infer U> ? U : T extends Record<string, AnySchema> ? InferFromObject<T> : T extends AnySchema[] ? InferFromObject<T> : never;
export declare type DeepPartial<T> = T extends Function ? T : T extends (infer U)[] ? DeepPartial<U>[] : T extends Record<string, unknown> ? {
    [K in keyof T]?: DeepPartial<T[K]>;
} : T | undefined;
//# sourceMappingURL=types.d.ts.map